/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

### File Specific:
   main-programmer: hdavid, amaechler
*/

#include "rain_effect.h"

#include "util/loading/load_param.h"
#include "util/loading/factory.h"

#include "glincl.h"

// *****

#include "material.h"
#include "state.h"
#include "shell_command.h"

#include "parser/tinyxml/tinyxml.h"
#include <algorithm>
#include "box_emitter.h"
#include "spark_particles.h"


using namespace std;

CREATE_FACTORY(RainEffect, CL_RAIN_EFFECT);

RainEffect::RainEffect(const TiXmlElement* root)
{
  this->setClassID(CL_RAIN_EFFECT, "RainEffect");

  // BoxEmitter::BoxEmitter(const Vector& size, float emissionRate, float velocity, float angle)

//         <name>RainEmitter</name>
//         <rel-coor>-1200, 300, 0</rel-coor>
//         <rel-dir>3.1412, 0, 0, 1</rel-dir>
//         <spread>0,.3</spread>
//         <emission-velocity>500,50</emission-velocity>
//         <size>1000, 1000</size>
//         <rate>5000</rate>

  this->emitter = new BoxEmitter(Vector(1000, 300, 100), 20000, 1000, M_2_PI);
  this->emitter->setRelCoor(100, 0, 0);
  // this->emitter->setParent(this);
  this->emitter->setSpread(0, 0.3);

  if (root != NULL)
    this->loadParams(root);

  this->activate();
}

RainEffect::~RainEffect()
{
  this->deactivate();
}


void RainEffect::loadParams(const TiXmlElement* root)
{
  WeatherEffect::loadParams(root);

//   LoadParam(root, "rel-coor", this, RainEffect, setRainMode);
// 
//   LoadParam(root, "rate", this, RainEffect, setRainColor);
//   LoadParam(root, "velocity", this, RainEffect, setRainColor);
//   LoadParam(root, "angle", this, RainEffect, setRainColor);
// 
//   LoadParam(root, "spread", this, RainEffect, setRainColor);

  // BoxEmitter::BoxEmitter(const Vector& size, float emissionRate, float velocity, float angle)

  this->emitter = new BoxEmitter(Vector(0, 1000, 0), 20000, 4000, 1);
  this->emitter->setRelCoor(0, 100, 0);
  // this->emitter->setParent(this);
  this->emitter->setSpread(0, 0.3);

// LOAD_PARAM_START_CYCLE(root, element);
//   {
//     element->ToText();
//     // PER-PARTICLE-ATTRIBUTES:
//     LoadParam_CYCLE(element, "radius", this, ParticleSystem, setRadius)
//     .describe("The Radius of each particle over time (TimeIndex [0-1], radius at TimeIndex, randomRadius at TimeIndex)");
// 
//     LoadParam_CYCLE(element, "mass", this, ParticleSystem, setMass)
//     .describe("The Mass of each particle over time (TimeIndex: [0-1], mass at TimeIndex, randomMass at TimeIndex)");
// 
//     LoadParam_CYCLE(element, "color", this, ParticleSystem, setColor)
//     .describe("The Color of each particle over time (TimeIndex: [0-1], red: [0-1], green: [0-1], blue: [0-1], alpha: [0-1])");
//   }
//   LOAD_PARAM_END_CYCLE(element);

}

bool RainEffect::init()
{}

SparkParticles* RainEffect::rainParticles = NULL;

bool RainEffect::activate()
{
//   PRINTF(0)( "Enabling Rain Effect, mode: %i, density: %f, start: %f, end: %f, color %f, %f, %f\n", this->rainMode, this->rainDensity, this->rainStart, this->rainEnd, this->colorVector.x, this->colorVector.y, this->colorVector.z);
	PRINTF(0)("Activating Rain Effect\n");

	if (unlikely(RainEffect::rainParticles == NULL))
	{
		RainEffect::rainParticles = new SparkParticles(10000);
		RainEffect::rainParticles->setName("RainParticles");
		RainEffect::rainParticles->setLifeSpan(2, 2);
		RainEffect::rainParticles->setRadius(0.05, 0.05);
		RainEffect::rainParticles->setRadius(0.04, 0.04);
		RainEffect::rainParticles->setRadius(0.03, 0.03);
		RainEffect::rainParticles->setColor(0, 0, 0, 1, .3);
		RainEffect::rainParticles->setColor(0.5, 0.5, 0.5, 1, 0.2);
		RainEffect::rainParticles->setColor(0.8, 0.8, 0.2, 0.3, 0.3);
		RainEffect::rainParticles->setColor(1, 0.5, 0.5, 1, 0);

//      <life-span>2,2</life-span>
//      <radius>0, 0.05, 0.05</radius>
//      <radius>1, 0.05, 0.05</radius>
//      <mass>0,1, .05</mass>
//      <color>0, 0, 0, 1, .3</color>
//      <color>.5, .5, .5, 1, .2</color>
//      <color>1,0.5, .5,1, .0</color>
//      <precache>2</precache>
//      <emitters>
//       <PlaneEmitter>

//       </PlaneEmitter>
//      </emitters>

	}

	this->emitter->setSystem(RainEffect::rainParticles);
//	this->size = 4.0;

	RainEffect::rainParticles->debug();
//	this->updateNode(0);
	this->emitter->setEmissionRate(5000.0);
	this->emitter->setEmissionVelocity(50.0);

//	this->setHealth(200);

}


bool RainEffect::deactivate()
{
  PRINTF(0)("Deactivating Rain Effect\n");

  this->emitter->setSystem(NULL);
 // this->lifeCycle = 0.0;
 // this->toList(OM_NULL);

//  GarbageCollector::getInstance()->collect(this);
//  this->toList(OM_DEAD);

}

void RainEffect::draw() const
{
//  Particle* drawPart = ParticleSystem::particles;

//   glDepthMask(GL_FALSE);
//   glPushAttrib(GL_ENABLE_BIT);
// 
//   glDisable(GL_LIGHTING);
//   glDisable(GL_TEXTURE_2D);
// 
//   glEnable(GL_LINE_SMOOTH);
//   glEnable(GL_BLEND);
//   glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
// 
//   glLineWidth(2.0);
//   glBegin(GL_LINES);
//   while (likely(drawPart != NULL))
//   {
//   //    printf("%f %f %f %f\n", drawPart->color[0], drawPart->color[1], drawPart->color[2], drawPart->color[3]);
//     glColor4fv(drawPart->color);
//     glVertex3f(drawPart->position.x,  drawPart->position.y,  drawPart->position.z);
//     glVertex3f(drawPart->position.x - drawPart->velocity.x * drawPart->radius,
//                drawPart->position.y - drawPart->velocity.y * drawPart->radius,
//                drawPart->position.z - drawPart->velocity.z * drawPart->radius);
//     drawPart = drawPart->next;
//   }
//   glEnd();
// 
//   glDepthMask(GL_TRUE);
//   glPopAttrib();
}
